import unittest
import collections

import MockMockMock
import MockMockMock._Details.ArgumentChecking

def isCallable( x ):
    return isinstance( x, collections.Callable )

class TestException( Exception ):
    pass

class PublicInterface( unittest.TestCase ):
    def setUp( self ):
        unittest.TestCase.setUp( self )
        self.mocks = MockMockMock.Engine()
        self.myMock = self.mocks.create( "myMock" )

    def testMockMockMock( self ):
        self.assertEqual( self.dir( MockMockMock ), [ "Engine", "Exception", "tests" ] )

    def testEngine( self ):
        self.assertEqual( self.dir( self.mocks ), [ "alternative", "atomic", "create", "optional", "ordered", "repeated", "tearDown", "unordered" ] )
        self.assertFalse( isCallable( self.mocks ) )

    def testMock( self ):
        self.assertEqual( self.dir( self.myMock ), [ "expect", "object" ] )
        self.assertFalse( isCallable( self.myMock ) )

    def testExpect( self ):
        self.assertEqual( self.dir( self.myMock.expect ), [] )
        self.assertTrue( isCallable( self.myMock.expect ) )

    def testExpectation( self ):
        self.assertEqual( self.dir( self.myMock.expect.foobar ), [ "andExecute", "andRaise", "andReturn", "withArguments" ] )
        self.assertTrue( isCallable( self.myMock.expect.foobar ) )

    def testCalledExpectation( self ):
        self.assertEqual( self.dir( self.myMock.expect.foobar( 42 ) ), [ "andExecute", "andRaise", "andReturn" ] )
        self.assertFalse( isCallable( self.myMock.expect.foobar( 42 ) ) )
        self.assertEqual( self.dir( self.myMock.expect.foobar.withArguments( 42 ) ), [ "andExecute", "andRaise", "andReturn" ] )
        self.assertFalse( isCallable( self.myMock.expect.foobar.withArguments( 42 ) ) )

    def testCalledThenAndedExpectation( self ):
        self.assertEqual( self.dir( self.myMock.expect.foobar( 42 ).andReturn( 12 ) ), [] )
        self.assertFalse( isCallable( self.myMock.expect.foobar( 42 ).andReturn( 12 ) ) )
        self.assertEqual( self.dir( self.myMock.expect.foobar( 42 ).andRaise( TestException() ) ), [] )
        self.assertFalse( isCallable( self.myMock.expect.foobar( 42 ).andRaise( TestException() ) ) )
        self.assertEqual( self.dir( self.myMock.expect.foobar( 42 ).andExecute( lambda : 12 ) ), [] )
        self.assertFalse( isCallable( self.myMock.expect.foobar( 42 ).andExecute( lambda : 12 ) ) )

    def testAndedExpectation( self ):
        self.assertEqual( self.dir( self.myMock.expect.foobar.andReturn( 12 ) ), [] )
        self.assertFalse( isCallable( self.myMock.expect.foobar.andReturn( 12 ) ) )
        self.assertEqual( self.dir( self.myMock.expect.foobar.andRaise( TestException() ) ), [] )
        self.assertFalse( isCallable( self.myMock.expect.foobar.andRaise( TestException() ) ) )
        self.assertEqual( self.dir( self.myMock.expect.foobar.andExecute( lambda : 12 ) ), [] )
        self.assertFalse( isCallable( self.myMock.expect.foobar.andExecute( lambda : 12 ) ) )

    def testObject( self ):
        ### @todo Maybe expose expected calls in myMock.object.__dir__
        self.assertEqual( self.dir( self.myMock.object ), [] )

    def dir( self, o ):
        return sorted( a for a in dir( o ) if not a.startswith( "_" ) )

class SingleExpectation( unittest.TestCase ):
    def setUp( self ):
        unittest.TestCase.setUp( self )
        self.mocks = MockMockMock.Engine()
        self.myMock = self.mocks.create( "myMock" )

    def tearDown( self ):
        self.mocks.tearDown()

    def testMethodCall( self ):
        self.myMock.expect.foobar()
        self.myMock.object.foobar()

    def testMethodCallWithSimpleArgument( self ):
        self.myMock.expect.foobar( 42 )
        self.myMock.object.foobar( 42 )

    def testMethodCallWithReturn( self ):
        returnValue = object()
        self.myMock.expect.foobar().andReturn( returnValue )
        # Not only "==" but "is"
        self.assertTrue( self.myMock.object.foobar() is returnValue )

    def testPropertyWithReturn( self ):
        self.myMock.expect.foobar.andReturn( 42 )
        self.assertEqual( self.myMock.object.foobar, 42 )

    def testObjectCallWithArgumentsAndReturn( self ):
        self.myMock.expect( 43, 44 ).andReturn( 42 )
        self.assertEqual( self.myMock.object( 43, 44 ), 42 )

    def testMethodCallWithRaise( self ):
        self.myMock.expect.foobar().andRaise( TestException() )
        with self.assertRaises( TestException ):
            self.myMock.object.foobar()

    def testPropertyWithRaise( self ):
        self.myMock.expect.foobar.andRaise( TestException() )
        with self.assertRaises( TestException ):
            self.myMock.object.foobar

    def testMethodCallWithSpecificAction( self ):
        self.check = False
        def f():
            self.check = True
        self.myMock.expect.foobar().andExecute( f )
        self.myMock.object.foobar()
        self.assertTrue( self.check )

    def testPropertyWithSpecificAction( self ):
        self.check = False
        def f():
            self.check = True
        self.myMock.expect.foobar.andExecute( f )
        self.myMock.object.foobar
        self.assertTrue( self.check )

class SingleExpectationNotCalled( unittest.TestCase ):
    def setUp( self ):
        unittest.TestCase.setUp( self )
        self.mocks = MockMockMock.Engine()
        self.myMock = self.mocks.create( "myMock" )

    def testMethodCallWithBadArgument( self ):
        self.myMock.expect.foobar( 42 )
        with self.assertRaises( MockMockMock.Exception ) as cm:
            self.myMock.object.foobar( 43 )
        self.assertEqual( cm.exception.message, "myMock.foobar called with bad arguments" )

    def testMethodCallWithBadName( self ):
        self.myMock.expect.foobar()
        with self.assertRaises( MockMockMock.Exception ) as cm:
            self.myMock.object.barbaz()
        self.assertEqual( cm.exception.message, "myMock.barbaz called instead of myMock.foobar" )

    def testMethodCallWithBadName2( self ):
        self.myMock.expect.foobar2()
        with self.assertRaises( MockMockMock.Exception ) as cm:
            self.myMock.object.barbaz()
        self.assertEqual( cm.exception.message, "myMock.barbaz called instead of myMock.foobar2" )

    def testPropertyWithBadName( self ):
        self.myMock.expect.foobar.andReturn( 42 )
        with self.assertRaises( MockMockMock.Exception ) as cm:
            self.myMock.object.barbaz
        self.assertEqual( cm.exception.message, "myMock.barbaz called instead of myMock.foobar" )

    def testMethodNotCalled( self ):
        self.myMock.expect.foobar()
        with self.assertRaises( MockMockMock.Exception ) as cm:
            self.mocks.tearDown()
        self.assertEqual( cm.exception.message, "myMock.foobar not called" )
        self.myMock.object.foobar()
        self.mocks.tearDown()

    def testPropertyNotCalled( self ):
        self.myMock.expect.foobar
        with self.assertRaises( MockMockMock.Exception ) as cm:
            self.mocks.tearDown()
        self.assertEqual( cm.exception.message, "myMock.foobar not called" )
        self.myMock.object.foobar
        self.mocks.tearDown()

class ExpectationSequence( unittest.TestCase ):
    def setUp( self ):
        unittest.TestCase.setUp( self )
        self.mocks = MockMockMock.Engine()
        self.myMock = self.mocks.create( "myMock" )

    def testTwoCalls( self ):
        self.myMock.expect.foobar()
        self.myMock.expect.barbaz()
        self.myMock.object.foobar()
        self.myMock.object.barbaz()
        self.mocks.tearDown()

    def testCallNotExpectedFirst( self ):
        self.myMock.expect.foobar()
        self.myMock.expect.barbaz()
        with self.assertRaises( MockMockMock.Exception ) as cm:
            self.myMock.object.barbaz()
        self.assertEqual( cm.exception.message, "myMock.barbaz called instead of myMock.foobar" )

    def testCallWithArgumentsNotExpectedFirst( self ):
        self.myMock.expect.foobar( 42 )
        self.myMock.expect.foobar( 43 )
        with self.assertRaises( MockMockMock.Exception ) as cm:
            self.myMock.object.foobar( 43 )
        self.assertEqual( cm.exception.message, "myMock.foobar called with bad arguments" )

    def testManyCalls( self ):
        self.myMock.expect.foobar( 1 )
        self.myMock.expect.foobar( 2 )
        self.myMock.expect.foobar( 3 )
        self.myMock.expect.foobar( 4 )
        self.myMock.expect.foobar( 5 )
        self.myMock.expect.foobar( 6 )
        self.myMock.object.foobar( 1 )
        self.myMock.object.foobar( 2 )
        self.myMock.object.foobar( 3 )
        self.myMock.object.foobar( 4 )
        self.myMock.object.foobar( 5 )
        self.myMock.object.foobar( 6 )
        self.mocks.tearDown()

class SequenceBetweenSeveralLinkedMocks( unittest.TestCase ):
    def setUp( self ):
        unittest.TestCase.setUp( self )
        self.mocks = MockMockMock.Engine()
        self.m1 = self.mocks.create( "m1" )
        self.m2 = self.mocks.create( "m2" )

    def testShortCorrectSequence( self ):
        self.m1.expect.foobar( 42 )
        self.m2.expect.foobar( 43 )
        self.m1.object.foobar( 42 )
        self.m2.object.foobar( 43 )
        self.mocks.tearDown()

    def testShortInvertedSequence( self ):
        self.m1.expect.foobar( 42 )
        self.m2.expect.foobar( 43 )
        with self.assertRaises( MockMockMock.Exception ) as cm:
            self.m2.object.foobar( 43 )
        self.assertEqual( cm.exception.message, "m2.foobar called instead of m1.foobar" )

class SequenceBetweenSeveralIndependentMocks( unittest.TestCase ):
    def setUp( self ):
        unittest.TestCase.setUp( self )
        self.mocks1 = MockMockMock.Engine()
        self.mocks2 = MockMockMock.Engine()
        self.m1 = self.mocks1.create( "m1" )
        self.m2 = self.mocks2.create( "m2" )

    def testSameOrderSequence( self ):
        self.m1.expect.foobar( 42 )
        self.m2.expect.foobar( 43 )
        self.m1.object.foobar( 42 )
        self.m2.object.foobar( 43 )
        self.mocks1.tearDown()
        self.mocks2.tearDown()

    def testOtherOrderSequence( self ):
        self.m1.expect.foobar( 42 )
        self.m2.expect.foobar( 43 )
        self.m2.object.foobar( 43 )
        self.m1.object.foobar( 42 )
        self.mocks1.tearDown()
        self.mocks2.tearDown()

class ArgumentCheckers( unittest.TestCase ):
    def testCheckerIsUsedByCall( self ):
        # We use a myMock...
        checker = MockMockMock.Engine().create( "checker" )
        checker.expect( ( 12, ), {} ).andReturn( True )
        checker.expect( ( 13, ), {} ).andReturn( False )
    
        # ...to test a mock!
        m = MockMockMock.Engine().create( "m" )
        m.expect.foobar.withArguments( checker.object ).andReturn( 42 )
        m.expect.foobar.withArguments( checker.object ).andReturn( 43 )
        self.assertEqual( m.object.foobar( 12 ), 42 )
        with self.assertRaises( MockMockMock.Exception ) as cm:
            m.object.foobar( 13 )
        self.assertEqual( cm.exception.message, "m.foobar called with bad arguments" )

    # def testIdentityChecker( self ):
    # def testTypeChecker( self ):
    # def testRangeChecker( self ):

    def testEqualityChecker( self ):
        c = MockMockMock._Details.ArgumentChecking.Equality( ( 1, 2, 3 ), { 1: 1, 2: 2, 3: 3 } )
        self.assertTrue( c( ( 1, 2, 3 ), { 1: 1, 2: 2, 3: 3 } ) )
        self.assertFalse( c( ( 1, 2, 3 ), { 1: 1, 2: 2, 3: 4 } ) )
        self.assertFalse( c( ( 1, 2, 4 ), { 1: 1, 2: 2, 3: 3 } ) )

class Ordering( unittest.TestCase ):
    def setUp( self ):
        unittest.TestCase.setUp( self )
        self.mocks = MockMockMock.Engine()
        self.myMock = self.mocks.create( "myMock" )

    def testUnorderedGroupOfSameMethod( self ):
        with self.mocks.unordered:
            self.myMock.expect.foobar( 1 ).andReturn( 11 )
            self.myMock.expect.foobar( 1 ).andReturn( 13 )
            self.myMock.expect.foobar( 2 ).andReturn( 12 )
            self.myMock.expect.foobar( 1 ).andReturn( 14 )
        self.assertEqual( self.myMock.object.foobar( 2 ), 12 )
        self.assertEqual( self.myMock.object.foobar( 1 ), 11 )
        self.assertEqual( self.myMock.object.foobar( 1 ), 13 )
        self.assertEqual( self.myMock.object.foobar( 1 ), 14 )
        self.mocks.tearDown()

    ### @todo Allow unordered property and method calls on the same name: difficult
    def testUnorderedGroupOfSameMethodAndProperty( self ):
        with self.assertRaises( MockMockMock.Exception ) as cm:
            with self.mocks.unordered:
                self.myMock.expect.foobar()
                self.myMock.expect.foobar
            self.myMock.object.foobar
        self.assertEqual( cm.exception.message, "myMock.foobar is expected as a property and as a method call in an unordered group" )

    def testUnorderedGroupOfSamePropertyAndMethod( self ):
        with self.assertRaises( MockMockMock.Exception ) as cm:
            with self.mocks.unordered:
                self.myMock.expect.foobar
                self.myMock.expect.foobar()
            self.myMock.object.foobar()
        self.assertEqual( cm.exception.message, "myMock.foobar is expected as a property and as a method call in an unordered group" )



class Mock( object ):
    def __init__( self, name, handler ):
        self.__name = name
        self.__handler = handler

    @property
    def expect( self ):
        return self.__handler.expect( self.__name )

    @property
    def object( self ):
        return self.__handler.object( self.__name )



from _Details.Mock import Mock
from _Details.ExpectationGrouping import OrderedExpectationGroup, UnorderedExpectationGroup, AtomicExpectationGroup, OptionalExpectationGroup, AlternativeExpectationGroup, RepeatedExpectationGroup
from _Details.ExpectationHandler import ExpectationHandler

class Engine:
    def __init__( self ):
        self.__handler = ExpectationHandler( OrderedExpectationGroup() )
        self.__mocks = list()

    def create( self, name ):
        mock = Mock( name, self.__handler )
        self.__mocks.append( mock )
        return mock

    def tearDown( self ):
        self.__handler.tearDown()

    @property
    def unordered( self ):
        return self.__handler.pushGroup( UnorderedExpectationGroup() )

    @property
    def ordered( self ):
        return self.__handler.pushGroup( OrderedExpectationGroup() )

    @property
    def atomic( self ):
        return self.__handler.pushGroup( AtomicExpectationGroup() )

    @property
    def optional( self ):
        return self.__handler.pushGroup( OptionalExpectationGroup() )

    @property
    def alternative( self ):
        return self.__handler.pushGroup( AlternativeExpectationGroup() )

    @property
    def repeated( self ):
        return self.__handler.pushGroup( RepeatedExpectationGroup() )



# Copyright 2012 Vincent Jacques
# vincent@vincent-jacques.net

import unittest

import AllTests


def run():
    testLoader = unittest.loader.TestLoader()
    testRunner = unittest.runner.TextTestRunner(verbosity=1)
    test = testLoader.loadTestsFromModule(AllTests)
    return testRunner.run(test)



from MockException import MockException
from Expectation import Expectation
import ArgumentChecking

class AnyAttribute:
    def __init__( self, apply, mockName ):
        self.__apply = apply
        self.__mockName = mockName

    def __getattr__( self, name ):
        if name == "__dir__": raise AttributeError()
        return self.__apply( self.__mockName + "." + name )

    def __call__( self, *args, **kwds ): # Needed to make isinstance( mock.expect, collections.Callable ) return True
        return self.__getattr__( "__call__" )( *args, **kwds )

class CallChecker:
    def __init__( self, handler, expectations ):
        self.__handler = handler
        self.__expectations = expectations

    def __call__( self, *args, **kwds ):
        return self.__handler.checkExpectationCall( self.__expectations, args, kwds )

class BasicExpectationProxy:
    def __init__( self, expectation ):
        self.__expectation = expectation

    def andReturn( self, value ):
        return self.andExecute( lambda : value )

    def andRaise( self, exception ):
        def Raise(): raise exception
        return self.andExecute( Raise )

    def andExecute( self, action ):
        self.__expectation.setAction( action )
        return None

class CallableExpectationProxy( BasicExpectationProxy ):
    def __init__( self, expectation ):
        BasicExpectationProxy.__init__( self, expectation )
        self.__expectation = expectation

    def __call__( self, *args, **kwds ):
        return self.withArguments( ArgumentChecking.Equality( args, kwds ) )

    def withArguments( self, checker ):
        self.__expectation.expectCall( checker )
        return BasicExpectationProxy( self.__expectation )

class ExpectationHandler( object ):
    def __init__( self, initialGroup ):
        self.__currentGroup = initialGroup

    # expect
    def expect( self, mockName ):
        return AnyAttribute( self.addExpectation, mockName )

    def addExpectation( self, name ):
        # Note that accepting name == "__call__" allows the mock object to be callable with no specific code
        expectation = Expectation( name )
        self.__currentGroup.addExpectation( expectation )
        return CallableExpectationProxy( expectation )

    def pushGroup( self, group ):
        self.__currentGroup.addGroup( group )
        self.__currentGroup = group
        return self.StackPoper( self )

    class StackPoper:
        def __init__( self, handler ): self.__handler = handler
        def __enter__( self ): pass
        def __exit__( self, *ignored ): self.__handler.popGroup()

    def popGroup( self ):
        self.__currentGroup = self.__currentGroup.parent

    # call
    def object( self, mockName ):
        return AnyAttribute( self.checkExpectation, mockName )

    def checkExpectation( self, calledName ):
        possibleExpectations = self.__currentGroup.getCurrentPossibleExpectations()
        goodNamedExpectations = [ expectation for expectation in possibleExpectations if expectation.checkName( calledName ) ]

        if len( goodNamedExpectations ) == 0:
            raise MockException( calledName + " called instead of " + " or ".join( e.name for e in possibleExpectations ) )

        allGoodNamedExpectationsExpectNoCall = not any( expectation.expectsCall() for expectation in goodNamedExpectations )
        allGoodNamedExpectationsExpectCall = all( expectation.expectsCall() for expectation in goodNamedExpectations )

        if allGoodNamedExpectationsExpectCall:
            return CallChecker( self, goodNamedExpectations )
        elif allGoodNamedExpectationsExpectNoCall:
            return self.__callExpectation( goodNamedExpectations[ 0 ] )
        else:
            raise MockException( calledName + " is expected as a property and as a method call in an unordered group" )

    def checkExpectationCall( self, expectations, args, kwds ):
        for expectation in expectations:
            if expectation.checkCall( args, kwds ):
                return self.__callExpectation( expectation )
        raise MockException( expectations[ 0 ].name + " called with bad arguments" )

    def __callExpectation( self, expectation ):
        returnValue, self.__currentGroup = expectation.call()
        return returnValue

    def tearDown( self ):
        requiredCalls = self.__currentGroup.nbRequiredCalls()
        if requiredCalls:
            raise MockException( ", ".join( self.__currentGroup.getRequiredCallsExamples() ) + " not called" )



import unittest

import MockMockMock

groupMakers = {
    "o" : lambda f: f.ordered,
    "u" : lambda f: f.unordered,
    "a" : lambda f: f.atomic,
    "p" : lambda f: f.optional,
    "l" : lambda f: f.alternative,
    "r" : lambda f: f.repeated,
}

def makeExpectations( mocks, mock, groups ):
    if len( groups ) > 0:
        group = groups[ 0 ]
        with groupMakers[ group ]( mocks ):
            mock.expect.foobar( group + "A" )
            mock.expect.foobar( group + "B" )
            makeExpectations( mocks, mock, groups[ 1: ] )
            mock.expect.foobar( group + "C" )
            mock.expect.foobar( group + "D" )

def testAllowedOrder( allowedOrder ):
    def test( self ):
        for argument in allowedOrder:
            self.call( argument )
        self.mocks.tearDown()
    return test

def testForbidenOrder( forbidenOrder ):
    def test( self ):
        for argument in forbidenOrder[ : -1 ]:
            self.call( argument )
        with self.assertRaises( MockMockMock.Exception ):
            self.call( forbidenOrder[ -1 ] )
    return test

def testTearDownError( forbidenOrder ):
    def test( self ):
        for argument in forbidenOrder:
            self.call( argument )
        with self.assertRaises( MockMockMock.Exception ):
            self.mocks.tearDown()
    return test

def makeTestCase( groups, allowedOrders, forbidenOrders, tearDownErrors ):
    groups = list( groups )
    allowedOrders = [
        [
            allowedOrder[ 2 * i : 2 * i + 2 ]
            for i in range( len( allowedOrder ) / 2 )
        ]
        for allowedOrder in allowedOrders
    ]
    forbidenOrders = [
        [
            forbidenOrder[ 2 * i : 2 * i + 2 ]
            for i in range( len( forbidenOrder ) / 2 )
        ]
        for forbidenOrder in forbidenOrders
    ]
    tearDownErrors = [
        [
            tearDownError[ 2 * i : 2 * i + 2 ]
            for i in range( len( tearDownError ) / 2 )
        ]
        for tearDownError in tearDownErrors
    ]

    class TestCase( unittest.TestCase ):
        def setUp( self ):
            unittest.TestCase.setUp( self )
            self.mocks = MockMockMock.Engine()
            self.myMock = self.mocks.create( "myMock" )
            makeExpectations( self.mocks, self.myMock, groups )

        def call( self, argument ):
            assert( len( argument ) == 2 )
            assert( argument[ 0 ] in groupMakers )
            assert( argument[ 1 ] in "ABCDX" )
            self.myMock.object.foobar( argument )

    for allowedOrder in allowedOrders:
        setattr( TestCase, "test_" + "".join( allowedOrder ), testAllowedOrder( allowedOrder ) )

    for forbidenOrder in forbidenOrders:
        setattr( TestCase, "test_" + "".join( forbidenOrder ), testForbidenOrder( forbidenOrder ) )

    for tearDownError in tearDownErrors:
        setattr( TestCase, "test_" + "".join( tearDownError ), testTearDownError( tearDownError ) )

    TestCase.__name__ = "TestCase_" + "".join( groups )
        
    return TestCase
    
UnorderedGroup = makeTestCase(
    "u",
    [
        # Completed in any order
        "uAuBuCuD",
        "uBuAuDuC",
        "uBuCuAuD",
    ],
    [
        # BaD argument
        "uAuX",
    ],
    [
        # Not completed
        "uAuBuC",
        "uAuB",
        "uA",
        "",
    ]
)

OrderedGroup = makeTestCase(
    "o",
    [
        # Completed in good order
        "oAoBoCoD",
    ],
    [
        # Wrong order
        "oB",
    ],
    [
        # Not completed
        "oAoBoC",
        "oAoB",
        "oA",
        "",
    ]
)

OptionalGroup = makeTestCase(
    "p",
    [
        # Completed in good order
        "pApBpCpD",
        # Not completed
        "pApBpC",
        "pApB",
        "pA",
        "",
    ],
    [
        # Wrong order
        "pB",
    ],
    [
    ]
)

AlternativeGroup = makeTestCase(
    "l",
    [
        # One call
        "lA",
        "lB",
        "lC",
        "lD",
    ],
    [
        # Two calls
        "lAlB",
        "lBlA",
    ],
    [
        # No call
        "",
    ]
)

RepeatedGroup = makeTestCase(
    "r",
    [
        # Zero call
        "",
        # One time
        "rArBrCrD",
        # Several times
        "rArBrCrDrArBrCrD",
        # "rArBrCrDrArBrCrDrArBrCrD",
    ],
    [
    ],
    [
        # Partial calls
        "rArB",
        # "rArBrCrDrArB",
        # "rArBrCrDrArBrCrDrArB",
    ]
)

OrderedInUnorderedGroup = makeTestCase(
    "uo",
    [
        # Original order
        "uAuBoAoBoCoDuCuD",
        # Other possible orders
        #  ordered group at once
        "oAoBoCoDuAuBuCuD",
        "uAuBuCuDoAoBoCoD",
        #  ordered group in pieces
        "oAuAoBuBoCuCoDuD",
        "oAuDoBuCoCuBoDuA",
    ],
    [
        # Ordered group in wrong order
        "uAuBoB",
    ],
    [
    ]
)

AtomicInUnorderedGroup = makeTestCase(
    "ua",
    [
        # Original order
        "uAuBaAaBaCaDuCuD",
        # Other possible orders
        #  atomic group at once
        "aAaBaCaDuAuBuCuD",
        "uAuBuCuDaAaBaCaD",
    ],
    [
        # Atomic group in wrong order
        "uAuBaB",
        # Atomic group in pieces
        "uAaAuB",
    ],
    [
    ]
)

UnorderedInRepeatedGroup = makeTestCase(
    "ru",
    [
        "",
        "rArBuAuBuCuDrCrD",
        "rArBuAuBuCuDrCrDrArBuAuBuCuDrCrD",
    ],
    [
        # Partial
        "rArBuAuBuCrC",
    ],
    [
        # Partial
        "rArBuAuBuCuDrC",
    ]
)



class Equality:
    def __init__( self, args, kwds ):
        self.__args = args
        self.__kwds = kwds

    def __call__( self, args, kwds ):
        return self.__args == args and self.__kwds == kwds



class Expectation( object ):
    def __init__( self, name ):
        self.name = name
        self.__checker = None
        self.__action = lambda : None

    # expect
    def expectCall( self, checker ):
        self.__checker = checker

    def setAction( self, action ):
        self.__action = action

    # check
    def checkName( self, name ):
        return self.name == name

    def expectsCall( self ):
        return self.__checker is not None

    def checkCall( self, args, kwds ):
        return self.__checker( args, kwds )

    # call
    def call( self ):
        return self.__action()



# Copyright 2012 Vincent Jacques
# vincent@vincent-jacques.net

# This file is part of PyGithub. http://vincent-jacques.net/PyGithub

# PyGithub is free software: you can redistribute it and/or modify it under the terms of the GNU Lesser General Public License
# as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.

# PyGithub is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Lesser General Public License for more details.

# You should have received a copy of the GNU Lesser General Public License along with PyGithub.  If not, see <http://www.gnu.org/licenses/>.

import sys
import unittest

import Framework
import AllTests


def main(argv):
    if "--record" in argv:
        Framework.activateRecordMode()
        argv = [arg for arg in argv if arg != "--record"]

    unittest.main(module=AllTests, argv=argv)


if __name__ == "__main__":
    main(sys.argv)



#!/usr/bin/env python

# Copyright 2012 Vincent Jacques
# vincent@vincent-jacques.net

from distutils.core import setup, Command
import textwrap
import sys
import glob

class test( Command ):
    user_options = []

    def initialize_options( self ):
        pass

    def finalize_options( self ):
        pass

    def run( self ):
        try:
            import coverage
            analyseCoverage = True
        except ImportError:
            print "Unable to import coverage. Running tests without coverage analysis"
            analyseCoverage = False
        if analyseCoverage:
            cov = coverage.coverage(branch=True)
            cov.start()

        import MockMockMock.tests
        ok = MockMockMock.tests.run().wasSuccessful()

        if analyseCoverage:
            cov.stop()
            for f in glob.glob( "MockMockMock/*.py" ):
                ok = ok and len( cov.analysis2( f )[ 3 ] ) == 0
            cov.report(file=sys.stdout, include="MockMockMock/*")
        if ok:
            exit( 0 )
        else:
            exit( 1 )

setup(
    name = "MockMockMock",
    version = "0.1.1",
    description = "My mock library. Don't use it (yet)",
    author = "Vincent Jacques",
    author_email = "vincent@vincent-jacques.net",
    url = "",
    long_description = textwrap.dedent( """\
    """ ),
    packages = [
        "MockMockMock",
        "MockMockMock._Details",
        "MockMockMock.tests",
    ],
    classifiers = [
        "Development Status :: 2 - Pre-Alpha",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Topic :: Software Development",
    ],
    cmdclass = { "test": test },
)



class OrderedOrderingPolicy:
    def getCurrentPossibleExpectations( self, expectations ):
        possible = []
        for expectation in expectations:
            possible += expectation.getCurrentPossibleExpectations()
            if expectation.nbRequiredCalls() > 0:
                break
        return possible

class UnorderedOrderingPolicy:
    def getCurrentPossibleExpectations( self, expectations ):
        possible = []
        for expectation in expectations:
            possible += expectation.getCurrentPossibleExpectations()
        return possible

class AllCompletionPolicy:
    def nbRequiredCalls( self, expectations ):
        required = 0
        for expectation in expectations:
            required += expectation.nbRequiredCalls()
        return required

    def acceptsMoreCalls( self, expectations ):
        return any( len(expectation.getCurrentPossibleExpectations() ) != 0 for expectation in expectations )

    def markExpectationCalled( self, expectations, expectation ):
        pass

class AnyCompletionPolicy:
    def nbRequiredCalls( self, expectations ):
        return 0

    def acceptsMoreCalls( self, expectations ):
        return True

    def markExpectationCalled( self, expectations, expectation ):
        pass

class ExactlyOneCompletionPolicy:
    def nbRequiredCalls( self, expectations ):
        required = 1
        for expectation in expectations:
            if len( expectation.getCurrentPossibleExpectations() ) == 0:
                required = 0
        return required

    def acceptsMoreCalls( self, expectations ):
        return self.nbRequiredCalls( expectations ) == 1

    def markExpectationCalled( self, expectations, expectation ):
        pass

class RepeatedCompletionPolicy:
    def nbRequiredCalls( self, expectations ):
        required = 0
        if expectations[ 0 ].called:
            for expectation in expectations:
                required += expectation.nbRequiredCalls()
        return required

    def acceptsMoreCalls( self, expectations ):
        return True

    def markExpectationCalled( self, expectations, expectation ):
        if expectation is expectations[ -1 ]:
            for e in expectations:
                e.resetCall()

class UnstickyStickynessPolicy:
    def sticky( self ):
        return False

class StickyStickynessPolicy:
    def sticky( self ):
        return True

class ExpectationWrapper( object ):
    def __init__( self, expectation ):
        self.__expectation = expectation
        self.__parent = None
        self.__called = False

    @property
    def name( self ):
        return self.__expectation.name

    def expectsCall( self ):
        return self.__expectation.expectsCall()

    def checkCall( self, args, kwds ):
        return self.__expectation.checkCall( args, kwds )

    def checkName( self, name ):
        return self.__expectation.checkName( name )

    def setParent( self, parent ):
        assert( self.__parent is None )
        self.__parent = parent

    def getCurrentPossibleExpectations( self ):
        if self.__called:
            return []
        else:
            return [ self ]

    def nbRequiredCalls( self ):
        if self.__called:
            return 0
        else:
            return 1

    @property
    def called( self ):
        return self.__called

    def call( self ):
        self.__called = True
        self.__parent.markExpectationCalled( self )
        return self.__expectation.call(), self.__parent.rewindGroups()

    def resetCall( self ):
        self.__called = False

class ExpectationGroup:
    def __init__( self, ordering, completion, stickyness ):
        self.__ordering = ordering
        self.__completion = completion
        self.__stickyness = stickyness
        self.__parent = None
        self.__expectations = []

    def setParent( self, parent ):
        assert( self.__parent is None )
        self.__parent = parent

    @property
    def parent( self ):
        assert( self.__parent is not None )
        return self.__parent

    def addExpectation( self, expectation ):
        wrapper = ExpectationWrapper( expectation )
        wrapper.setParent( self )
        self.__expectations.append( wrapper )

    def addGroup( self, group ):
        group.setParent( self )
        self.__expectations.append( group )

    def getCurrentPossibleExpectations( self ):
        if self.__completion.acceptsMoreCalls( self.__expectations ):
            return self.__ordering.getCurrentPossibleExpectations( self.__expectations )
        else:
            return []

    def nbRequiredCalls( self ):
        return self.__completion.nbRequiredCalls( self.__expectations )

    def getRequiredCallsExamples( self ):
        ### @todo Implement
        return [ "myMock.foobar" ]

    def rewindGroups( self ):
        if self.__shallStick():
            return self
        else:
            return self.__parent.rewindGroups()

    def __shallStick( self ):
        if self.__parent is None:
            return True
        if self.__stickyness.sticky():
            return len( self.getCurrentPossibleExpectations() ) != 0
        return False

    def markExpectationCalled( self, expectation ):
        self.__completion.markExpectationCalled( self.__expectations, expectation )

    def resetCall( self ):
        for e in self.__expectations:
            e.resetCall()

def makeGroup( ordering, completion, stickyness ):
    class Group( ExpectationGroup ):
        def __init__( self ):
            ExpectationGroup.__init__( self, ordering(), completion(), stickyness() )

    return Group

OrderedExpectationGroup = makeGroup( OrderedOrderingPolicy, AllCompletionPolicy, UnstickyStickynessPolicy )
UnorderedExpectationGroup = makeGroup( UnorderedOrderingPolicy, AllCompletionPolicy, UnstickyStickynessPolicy )
AtomicExpectationGroup = makeGroup( OrderedOrderingPolicy, AllCompletionPolicy, StickyStickynessPolicy )
OptionalExpectationGroup = makeGroup( OrderedOrderingPolicy, AnyCompletionPolicy, UnstickyStickynessPolicy )
AlternativeExpectationGroup = makeGroup( UnorderedOrderingPolicy, ExactlyOneCompletionPolicy, UnstickyStickynessPolicy )
RepeatedExpectationGroup = makeGroup( OrderedOrderingPolicy, RepeatedCompletionPolicy, UnstickyStickynessPolicy )
